/*
  Custom CKEditor configurations
*/

CKEDITOR.editorConfig = function( config )
{
  // config.styleSet is an array of objects that define each style available
  // in the font styles tool in the ckeditor toolbar
  config.stylesSet =
  [
        /* Block Styles */

        // Each style is an object whose properties define how it is displayed
        // in the dropdown, as well as what it outputs as html into the editor
        // text area.
        { name : 'Paragraph'   , element : 'p' },
        { name : 'Heading 2'   , element : 'h2' },
        { name : 'Heading 3'   , element : 'h3' },
        { name : 'Heading 4'   , element : 'h4' },
        { name : 'Float Right', element : 'div', attributes : { 'style' : 'float:right;' } },
        { name : 'Float Left', element : 'div', attributes : { 'style' : 'float:left;' } },
        { name : 'Preformatted Text', element : 'pre' },
  ];

}

// When opening a dialog, a "definition" is created for it. For
// each editor instance the "dialogDefinition" event is then
// fired. We can use this event to make customizations to the
// definition of existing dialogs.
CKEDITOR.on( 'dialogDefinition', function( event ) {

  // Take the dialog name
  var dialogName = event.data.name;

  // The definition holds the structured data that is used to eventually
  // build the dialog and we can use it to customize just about anything.
  // In Drupal terms, it's sort of like CKEditor's version of a Forms API and
  // what we're doing here is a bit like a hook_form_alter.
  var dialogDefinition = event.data.definition;

  // Uncomment to print the dialogDefinition to the console
  // console.log(dialogDefinition);

  // Check if the definition is from the dialog we're
  // interested in (the "table" dialog).
  if ( dialogName == 'table' ) {

    // .getContents() returns an object reference to a set of fields in the
    // dialog, also referred to as tabs. The Table dialog has two tabs:
    // "Table Properties" and "Advanced". Each of those has an id. In this case,
    // the id we're interested in is 'info' for the Table Properties tab.
    var infoTab = dialogDefinition.getContents( 'info' );

    // Once we have the tab reference, we can use the object's .get() method
    // to get another object reference, this time to the field we want to change
    // Fields also have ids. The border field id is "txtBorder"
    var borderSizeField = infoTab.get("txtBorder");

    // Set the border to 0 (who uses html table borders anyway?)
    borderSizeField['default'] = 0;
  }
});



// Listen for the "pluginsLoaded" event, so we are sure that the
// "dialog" plugin has been loaded and we are able to do our
// customizations. We're going to do this for every instance of CKEditor
// but technically you could only do it for certain ones
for (var editorId in CKEDITOR.instances) {

  // Get a reference to the editor
  var editor = CKEDITOR.instances[editorId];

  // Add the even listener with the editor's .on() function
  editor.on('pluginsLoaded', function(ev) {

    // If our custom dialog has not been registered, do that now.
    if (!CKEDITOR.dialog.exists('youtubeDialog')) {

      // Register the dialog. The actual dialog definition is below
      CKEDITOR.dialog.add('youtubeDialog', ytDialogDefinition);
    }

    // Now that CKEditor knows about our dialog, we can create a
    // command that will open it
    editor.addCommand('youtubeDialogCmd', new CKEDITOR.dialogCommand( 'youtubeDialog' ));

    // Finally we can assign the command to a new button that we'll call youtube
    // Don't forget, the button needs to be assigned to the toolbar
    editor.ui.addButton( 'YouTube',
      {
        label : 'You Tube',
        command : 'youtubeDialogCmd'
      }
    );
  });
}

/*
  Our dialog definition. Here, we define which fields we want, we add buttons
  to the dialog, and supply a "submit" handler to process the user input
  and output our youtube iframe to the editor text area.
*/
var ytDialogDefinition = function (editor) {

  var dialogDefinition =
  {
    title : 'YouTube Embed',
    minWidth : 390,
    minHeight : 130,
    contents : [
      {
        // To make things simple, we're just going to have one tab
        id : 'tab1',
        label : 'Settings',
        title : 'Settings',
        expand : true,
        padding : 0,
        elements :
        [
          {
            // http://docs.cksource.com/ckeditor_api/symbols/CKEDITOR.dialog.definition.vbox.html
            type: 'vbox',
            widths : [ null, null ],
            styles : [ 'vertical-align:top' ],
            padding: '5px',
            children: [
              {
                // http://docs.cksource.com/ckeditor_api/symbols/CKEDITOR.dialog.definition.html.html
                type : 'html',
                padding: '5px',
                html : 'You can find the youtube video id in the url of the video. <br/> e.g. http://www.youtube.com/watch?v=<strong>VIDEO_ID</strong>.'
              },
              {
                // http://docs.cksource.com/ckeditor_api/symbols/CKEDITOR.dialog.definition.textInput.html
                type : 'text',
                id : 'txtVideoId',
                label: 'YouTube Video ID',
                style: 'margin-top:5px;',
                'default': '',
                validate: function() {
                  // Just a little light validation
                  // 'this' is now a CKEDITOR.ui.dialog.textInput object which
                  // is an extension of a CKEDITOR.ui.dialog.uiElement object
                  var value = this.getValue();
                  value = value.replace(/http:.*youtube.*?v=/, '');
                  this.setValue(value);
                },
                // The commit function gets called for each form element
                // when the dialog's commitContent Function is called.
                // For our dialog, commitContent is called when the user
                // Clicks the "OK" button which is defined a little further down
                commit: commitValue
              },
            ]
          },
          {
            // http://docs.cksource.com/ckeditor_api/symbols/CKEDITOR.dialog.definition.hbox.html
            type: 'hbox',
            widths : [ null, null ],
            styles : [ 'vertical-align:top' ],
            padding: '5px',
            children: [
              {
                type : 'text',
                id : 'txtWidth',
                label: 'Width',
                // We need to quote the default property since it is a reserved word
                // in javascript
                'default': 500,
                validate : function() {
                  var pass = true,
                    value = this.getValue();
                  pass = pass && CKEDITOR.dialog.validate.integer()( value )
                    && value > 0;
                  if ( !pass )
                  {
                    alert( "Invalid Width" );
                    this.select();
                  }
                  return pass;
                },
                commit: commitValue
              },
              {
                type : 'text',
                id : 'txtHeight',
                label: 'Height',
                'default': 300,
                validate : function() {
                  var pass = true,
                    value = this.getValue();
                  pass = pass && CKEDITOR.dialog.validate.integer()( value )
                    && value > 0;
                  if ( !pass )
                  {
                    alert( "Invalid Height" );
                    this.select();
                  }
                  return pass;
                },
                commit: commitValue
              },
              {
                // http://docs.cksource.com/ckeditor_api/symbols/CKEDITOR.dialog.definition.checkbox.html
                type : 'checkbox',
                id : 'chkAutoplay',
                label: 'Autoplay',
                commit: commitValue
              }
            ]
          }
        ]
      }
    ],

    // Add the standard OK and Cancel Buttons
    buttons : [ CKEDITOR.dialog.okButton, CKEDITOR.dialog.cancelButton ],

    // A "submit" handler for when the OK button is clicked.
    onOk : function() {

      // A container for our field data
      var data = {};

      // Commit the field data to our data object
      // This function calls the commit function of each field element
      // Each field has a commit function (that we define below) that will
      // dump it's value into the data object
      this.commitContent( data );

      if (data.info) {
        var info = data.info;
        // Set the autoplay flag
        var autoplay = info.chkAutoplay ? 'autoplay=1': 'autoplay=0';
        // Concatenate our youtube embed url for the iframe
        var src = 'http://youtube.com/embed/' + info.txtVideoId + '?' + autoplay;
        // Create the iframe element
        var iframe = new CKEDITOR.dom.element( 'iframe' );
        // Add the attributes to the iframe.
        iframe.setAttributes({
          'width': info.txtWidth,
          'height': info.txtHeight,
          'type': 'text/html',
          'src': src,
          'frameborder': 0
        });
        // Finally insert the element into the editor.
        editor.insertElement(iframe);
      }

    }
  };

  return dialogDefinition;
};

// Little helper function to commit field data to an object that is passed in:
var commitValue = function( data ) {
  var id = this.id;
  if ( !data.info )
    data.info = {};
  data.info[id] = this.getValue();
};
